/***************************************************************************
                          treeclecb.cpp  -  description
                             -------------------
    begin                : Sun Jul 14 2002
    copyright            : (C) 2002 by Kartik Patel
    email                : k_patel@vsnl.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "Fl_Editor.H"
#include <FL/Fl_File_Chooser.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Help_Dialog.H>

#include "treecle.h"

extern Fl_Pixmap *folderSmall;
extern Fl_Pixmap *fileSmall;
extern Fl_Pixmap *closedfolderSmall;
extern Fl_Help_Dialog helpwindow;

char *editorbuffer;//for the editor
char *chosenfilename;//for the data file
char messagebuffer[128];//for the message window
char datafilename[128];//for the data file

int datafileloaded = 0;//flag
int treemodified = 0;//flag
int treeopen = 1;//flag

FILE *datafilepointer;

Fl_Toggle_Node *currentnode;//node which is currently highlighted

void geteditordata (void);//assign data from the editor to the category
void savetree (Fl_Toggle_Node *sel);//save tree
void loadtree (Fl_Toggle_Node *sel, int, int);//read tree
void treecb (Fl_Widget *, void *);//callback for handling mouseclick on tree
void menucb (Fl_Widget *, void *);//menu callback
void searchcb (Fl_Button *, void *);//search button callback

void treecb (Fl_Toggle_Tree *o,  void *data)
{
Fl_Toggle_Node *sel;
Fl_Toggle_Tree *tr;
int i, n;
	
	strcpy (messagebuffer, "\0");
	messagewindow->value(messagebuffer);
	
	tr = (Fl_Toggle_Tree*) o;
	n = tr->selection_count();
	i = itemeditor->changed ();
	if (n)//continue
		;
	else//empty treelist! This kludge prevents a segfault.
		return;
		
	if (currentnode) {//assign editor contents to previous currentnode
		if (i) {
		//if (itemeditor->CharacterCount ()) {
			treemodified = 1;//tree needs saving
			geteditordata();
			currentnode->user_data (editorbuffer);
			strcpy (messagebuffer, "\0");
			sprintf (messagebuffer, "Assigned editor contents to previous category.");
		}
	}
	else {//no previous node; assign editor contents to this node
		if (tr->selected()) {//single selection
			currentnode = tr->selected();
			if (i && itemeditor->CharacterCount ()) {
				treemodified = 1;//tree needs saving
				geteditordata();
				currentnode->user_data (editorbuffer);
				strcpy (messagebuffer, "\0");
				sprintf (messagebuffer, "Assigned editor contents to this category.");
				messagewindow->value(messagebuffer);
				return;
			}
			else {//display contents of this node
				if ((char *) currentnode->user_data()) {
					itemeditor->Clear ();
					itemeditor->LoadFrom ((char *) currentnode->user_data());
				}
			}
		}
	}

	if (tr->selected()) {//an item or category is selected
		//work with local variable
		sel = (Fl_Toggle_Node*) tr->selected();
		//update currentnode pointer
		currentnode = (Fl_Toggle_Node*) sel;
		
		if ((char *) sel->user_data()) {
			itemeditor->Clear ();
			itemeditor->LoadFrom ((char *) sel->user_data());
		}
	}
	else {//tr->selected()=0 means we have a multiple select
		//clear currentnodepointer
		currentnode = NULL;
		itemeditor->Clear ();
		strcpy (messagebuffer, "\0");
		sprintf (messagebuffer, "Selected %i categories or items\0", n);
		messagewindow->value(messagebuffer);
	}
}
	
void menucb (Fl_Menu_ *o, void *data)
{
const Fl_Menu_Item *m;
Fl_Menu_ *mw;
int i;
Fl_File_Chooser *fc;
Fl_Toggle_Tree *tr;
Fl_Toggle_Node *sel;

        mw = (Fl_Menu_ *)o;
        m = mw->mvalue ();
	tr = (Fl_Toggle_Tree *) treelist;
	
        if (m == newmenu) {
        	if (treemodified) {
			int answer = 1;
			messagewindow->value("Data needs to be saved!");
			answer = fl_choice ("The data has been modified! Do you wish to save?", "Return to program", "Save", "Discard without saving");//0, 1, 2
			if (answer == 1) {//Yes
				mainmenu->value (3);//set menu item value = "Save"
				menucb (mainmenu, treelist);
			}
			else if (answer == 0)//Cancel
				return;
        	}
       		while (sel = tr->traverse_start ()) {//delete all nodes
       			tr->remove (sel);
       		}
        	
        	tr->redraw ();
        	itemeditor->Clear ();
        	currentnode = NULL;
       		datafileloaded = 0;
       		treemodified = 0;
        	basewindow->label ("Treecle");
        }
        if (m == openmenu) {
        	if (treemodified) {
			int answer = 1;
			messagewindow->value("Data needs to be saved!");
			answer = fl_choice ("The data has been modified! Do you wish to save?", "Return to program", "Save", "Discard without saving");//0, 1, 2
			if (answer == 0) {//Cancel
				return;
			}
			else if (answer == 1) {//Yes
				mainmenu->value (3);//set menu item value = "Save"
				menucb (mainmenu, treelist);
			}
       		}
		while (sel = tr->traverse_start ()) {//delete all nodes
			tr->remove (sel);
		}
       		
       		datafileloaded = 0;
       		treemodified = 0;

       		itemeditor->Clear ();

        	chosenfilename = fl_file_chooser("Open and load Treecle file", "*.trc", NULL, 0);
		if (chosenfilename == NULL)
			return;
		strcpy (datafilename, chosenfilename);
		
		datafilepointer = fopen (datafilename, "r");
		loadtree (0, 0, 1);
		fclose (datafilepointer);
		
		strcpy (messagebuffer, "Loaded ");
		strcat (messagebuffer, datafilename);
		messagewindow->value (messagebuffer);
		
		strcpy (messagebuffer, "Treecle -");
		strcat (messagebuffer, datafilename);
		basewindow->label (messagebuffer);
		datafileloaded = 1;//file name has been set
		
       		currentnode = treelist->traverse_start();
		treelist->select_range (currentnode, currentnode);
		itemeditor->Clear ();
		itemeditor->LoadFrom ((char *) currentnode->user_data());
		treelist->redraw ();		
        }
        if (m == savemenu) {
        	if (!treemodified) {
			strcpy (messagebuffer, "No modification to data. Nothing to save.");
			messagewindow->value (messagebuffer);
        		return;
        	}
		if (!datafileloaded) {
        		chosenfilename = fl_file_chooser("Save Treecle file", "*.trc", NULL, 0);
		}
		if (chosenfilename == NULL)
			return;
		
		strcpy (datafilename, chosenfilename);
		strcpy (messagebuffer, "Saved ");
		strcat (messagebuffer, datafilename);
		messagewindow->value (messagebuffer);
		
		strcpy (messagebuffer, "Treecle -");
		strcat (messagebuffer, datafilename);
		basewindow->label (messagebuffer);

		datafilepointer = fopen (datafilename, "w");
		sel = tr->traverse_start ();//start from beginning
		savetree (sel);
		fclose (datafilepointer);
		
		treemodified = 0;//tree does not need saving
		datafileloaded = 1;//file name has been set
        }
        if (m == saveasmenu) {
        	if (!treemodified) {
			strcpy (messagebuffer, "No modification to data. Nothing to save.");
			messagewindow->value (messagebuffer);
        		return;
        	}
        	
		chosenfilename = fl_file_chooser("Save Treecle file under new name", "*.trc", NULL, 0);		
		if (chosenfilename == NULL)
			return;
		
		strcpy (datafilename, chosenfilename);
		messagewindow->value (datafilename);
		
		datafilepointer = fopen (datafilename, "w");
		sel = tr->traverse_start ();//start from beginning
		savetree (sel);
		fclose (datafilepointer);
		
		strcpy (messagebuffer, "Saved ");
		strcat (messagebuffer, datafilename);
		messagewindow->value (messagebuffer);
				
		strcpy (messagebuffer, "Treecle -");
		strcat (messagebuffer, datafilename);
		basewindow->label (messagebuffer);
		
		treemodified = 0;//tree does not need saving
		datafileloaded = 1;//file name has been set
        }
        if (m == exitmenu) {
        	if (!treemodified)
			exit (0);
		else {
			int answer = 1;
			messagewindow->value("Data needs to be saved!");
			answer = fl_choice ("The data has been modified! Do you wish to save?", "Return to program", "Save and exit", "Exit without saving");//0, 1, 2
			if (answer == 2)//No
				exit (0);
			else if (answer == 1) {//Yes
				mainmenu->value (3);//set menu item value = "Save"
				menucb (mainmenu, treelist);
				mainmenu->value (5);//set menu item value = "Exit"
				menucb (mainmenu, treelist);
			
			}
			else//Cancel
				return;
		}
        }
        if (m == unselectmenu) {
		butcb (unselectbut, treelist);
        }
        if (m == addcatmenu) {
        	butcb (addcatbut, treelist);
        }
        if (m == additemmenu) {
        	butcb (additembut, treelist);
        }
        if (m == opencatitem) {
        	butcb (openallbut, treelist);
        }
        if (m == closecatitem) {
        	butcb (closeallbut, treelist);
        }
        if (m == deletecatmenu) {
        	butcb (deletecatbut, treelist);
        }
        if (m == sortmenu) {
        	butcb (sortbut, treelist);
        }
        if (m == helpmenu) {
        	if (!helpwindow.visible ())
        		helpwindow.show ();
        }
        if (m == aboutmenu) {
		aboutwindow->show ();
        }
}

void butcb (Fl_Button *o, Fl_Toggle_Tree *data)
{
static int i = 0, j = 0, k = 0, l = 0;
char catlabel[25];
char catdata[25];
char itemlabel[25];
char itemdata[25];

Fl_Toggle_Tree* tr;
Fl_Toggle_Node* sel;

	tr = data;//treelist
	if (o == openfilebut) {
		mainmenu->value (2);//set menu item value = "Open"
		menucb (mainmenu, treelist);
	}
	if (o == savefilebut) {
		mainmenu->value (3);//set menu item value = "Save"
		menucb (mainmenu, treelist);
	}
	if (o == addcatbut) {
		treemodified = 1;//tree needs saving
		++i;
		++j;
		sprintf (catdata, "Data for Category%i", i);
		sprintf (catlabel, "Category%i name", j);
		
		sel = tr->selected();
		if (sel == 0) {//multiple selection
			tr->add_next(catdata, catlabel, 1, folderSmall);
			messagewindow->value ("Added new category.");
			return;
		}
		//multiple selection, so sel = 0
		tr->traverse_start(sel);//beginning of tree
		if (sel->is_open() && sel->can_open())
			tr->add_sub(catdata, catlabel, 1, folderSmall);
		else
			tr->add_next(catdata, catlabel, 1, folderSmall);
		messagewindow->value ("Added new category.");
		return;
	}
	if (o == deletecatbut) {
		treemodified = 1;//tree needs saving
		int n = 1;
		if (tr->selected())//single category or item
			tr->remove(tr->selected());
		else {//multiple selection
			Fl_Toggle_Node* s;
			n = tr->selection_count();
			while ((s = tr->selection()))
				tr->remove(s);
		}
		if (tr->selected()) {//show the next category or item data
			sel = (Fl_Toggle_Node*) tr->selected();
			if ((char *) sel->user_data()) {
				itemeditor->Clear ();
				itemeditor->LoadFrom  ((char *) sel->user_data());
			}
		}
		sprintf (messagebuffer, "%i categories or items deleted!\0", n);
		messagewindow->value (messagebuffer);
		tr->redraw();
		currentnode = NULL;
	}
	if (o == additembut) {
		treemodified = 1;//tree needs saving
		++k;
		++l;
		sprintf (itemdata, "Data for Item%i", k);
		sprintf (itemlabel, "Item%i label", l);
		
		sel = (Fl_Toggle_Node*) tr->selected();
		if (sel == 0) {//no category selected
			tr->add_next(itemdata, itemlabel, 0, fileSmall);
			return;
		}

		tr->traverse_start(sel);//something selected
		if (sel->is_open() && sel->can_open()) {
			tr->add_sub(itemdata, itemlabel, 0, fileSmall);
		}
		else
			tr->add_next(itemdata, itemlabel, 0, fileSmall);
		messagewindow->value ("Added new data item.");
	}
	if (o == unselectbut) {
		tr->unselect ();
		tr->redraw();
		messagewindow->value ("Selection cleared.");
		currentnode = NULL;
	}
	if (o == openallbut) {
		sel = tr->traverse_start ();//start from beginning
		while (sel) {//until the end
			if (!sel->is_open () && sel->can_open())//category
				tr->open (sel);//so open it
			else//item
				sel = sel->next();
		}
		//itemeditor->Clear ();
		tr->unselect ();
		tr->redraw ();
		messagewindow->value ("Opened all categories. Nothing selected.");
		treeopen = 1;
	}
	if (o == closeallbut) {
		sel = tr->traverse_start ();//start from beginning
		while (sel) {//until the end
			if (sel->is_open () && sel->can_open())//category
				tr->close (sel);//so close it
			else//item
				sel = sel->next();
		}
		//itemeditor->Clear ();
		tr->unselect ();
		tr->redraw ();
		messagewindow->value ("Closed all categories. Nothing selected.");
		treeopen = 0;
	}
	if (o == sortbut) {
		treemodified = 1;//tree needs saving
		tr->sort_tree(Fl_Toggle_Tree::sort_by_label);
		tr->redraw ();
	}

}

void searchcb (Fl_Button *o, void *data)
{
const char *findstr;
char *datahit, *labelhit;
static Fl_Toggle_Node *foundnode;
	
	if (!treeopen)
		butcb (openallbut, treelist);
	findstr = searchtext->value();
	if (!foundnode)
		foundnode = treelist->traverse_start ();
	else
		foundnode = treelist->traverse_forward ();
	while (foundnode) {
		datahit = strstr ((char *) foundnode->user_data(), findstr);
		labelhit = strstr ((char *) foundnode->label (), findstr);
		if (datahit || labelhit) {
			itemeditor->Clear ();
			itemeditor->LoadFrom  ((char *) foundnode->user_data());
			treelist->select_range (foundnode, foundnode);
			treelist->redraw();
			currentnode = foundnode;
			return;
		}
		else {
			foundnode = treelist->traverse_forward ();
		}
	}
}

void geteditordata (void)
{
int i = 0, ccount;

	//see if there are any characters typed in
	if (itemeditor->CharacterCount()) {//something in the editor: get size
		ccount = itemeditor->GetTextBufferSize();
		if (editorbuffer) {//reallocate count bytes
			editorbuffer = (char *) realloc (editorbuffer, ccount);
			if (!editorbuffer)//exit on failure
				exit (1);
		}
		else
			editorbuffer = (char *) calloc (1, ccount);
		strcat (editorbuffer, "\0");
		itemeditor->SaveTo (editorbuffer);
	}

	else {//nothing in the editor
		if (editorbuffer) {
			free (editorbuffer);//free the buffer
			editorbuffer = NULL;
		}
	}

}

