/***************************************************************************
                          treeclefile.cpp  -  description
                             -------------------
    begin                : Fri Oct 4 2002
    copyright            : (C) 2002 by Kartik Patel
    email                : k_patel@vsnl.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "treecle.h"

extern Fl_Pixmap *folderSmall;
extern Fl_Pixmap *fileSmall;
extern FILE *datafilepointer;

void savetree (Fl_Toggle_Node *sel);//save tree
void loadtree (Fl_Toggle_Node *sel, int, int);//read tree
void readrecord (char *labelbuf, char *databuf,
		int *i, int *j, int *k, int *l, int *m);//read one node
int writerecord (Fl_Toggle_Node *s);//write one node

void loadtree (Fl_Toggle_Node *sel, int subnode, int nxnode)
{
Fl_Toggle_Node *s;
char *labelbuf, *databuf;
int labelsize, datasize, dummy;
int i, j, k, l, m, sizeread;
//open, child, next, prev, parent

//Each node is stored in the following format:
//    i            j        k        l       m   labelcount datacount labelbuf databuf
//can-open-int child-int next-int prev-int up-int   int        int      char*    char*
//  0 or 1	0 or 1	  0 or 1  0 or 1   0 or 1

	fscanf (datafilepointer, "%i%i%i%i%i%i%i",
	&i, &j, &k, &l, &m, &labelsize, &datasize);
		
	//allocate the required memory to buffer; count+1 for NULL
	labelbuf = (char *) calloc (1, labelsize+2);
	sizeread = fread (labelbuf, labelsize+1, 1, datafilepointer);
	strcat (labelbuf, "\0");
	if (!sizeread) {
		messagewindow->value ("Error in reading label.");
		return;
	}
	
	//allocate the required memory to buffer; count+1 for NULL
	databuf = (char *) calloc (1, datasize+1);
	sizeread = fread (databuf, datasize, 1, datafilepointer);
	strcat (databuf, "\0");
	if (!sizeread) {
		messagewindow->value ("Error in reading data.");
		return;
	}
	
	//i = 1: Openable category; i = 0: Unopenable item	
	if (nxnode) {//add a node on the same level
		if (i)//openable
			s = treelist->add_next (databuf, labelbuf, i, folderSmall);
		if (!i)
			s = treelist->add_next (databuf, labelbuf, i, fileSmall);
	}
	else {//add a sub node
		if (i)
			s = treelist->add_sub (databuf, labelbuf, i, folderSmall);
		if (!i)
			s = treelist->add_sub (databuf, labelbuf, i, fileSmall);
	}
	
	if (j) {//child exists, so follow this trail first
		treelist->traverse_start (s);
		loadtree (s, 1, 0);
	}
	if (k) {//not a child node, check for next node
		treelist->traverse_start (s);
		loadtree (s, 0, 1);
	}
	
	return;
}

void savetree (Fl_Toggle_Node *sel)
{
Fl_Toggle_Node *s;
int i = 0;

	i = writerecord (sel);
	if (i)//error in writing
		return;
		
	if (sel->child ()) {//go deeper down the tree to child node
		s = sel->child ();
		savetree (s);
	}
	if (sel->next()) {//go across to the next node at this level
		s = sel->next ();
		savetree (s);
	}
	fflush (datafilepointer);
	return;
}

int writerecord (Fl_Toggle_Node *sel)
{
int labelsize, datasize;
int i = 0, j = 0, k = 0, l = 0, m = 0, sizewritten;
char *labelbuffer = 0, *databuffer = 0;

//Each node is stored in the following format:
//    i            j        k        l       m
//can-open-int child-int next-int prev-int up-int
//  0 or 1	0 or 1	  0 or 1  0 or 1   0 or 1
//labecount label datacount data
//  int     char*    int    char*
	
	//assign the node attributes
	i = sel->can_open ();//this node is openable
	if (sel->child())//child exists
		j = 1;
	if (sel->next())//next node exists
		k = 1;
	if (sel->previous ())//previous node exists
		l = 1;
	if (sel->parent ())//parent node exists
		m = 1;
		
	//get the size of the node label, without the end NULL
	labelsize = strlen (sel->label());
	//get the size of the user-data, without the end NULL
	datasize = strlen ((char *)sel->user_data());
	//write the attributes of this node
	fprintf (datafilepointer, " %i %i %i %i %i %i %i ",
		i, j, k, l, m, labelsize, datasize);
		
	//allocate the required memory to buffer; count+1 for NULL
	labelbuffer = (char *) calloc (1, labelsize+1);
	strcpy (labelbuffer, sel->label());
	sizewritten = fwrite (labelbuffer, labelsize, 1, datafilepointer);
	if (!sizewritten) {
		messagewindow->value ("Error in writing label.");
		return 1;
	}
	
	//allocate the required memory to buffer; count+1 for NULL
	databuffer = (char *) calloc (1, datasize+1);
	strcpy (databuffer, (char *) sel->user_data());
	sizewritten = fwrite (databuffer, datasize, 1, datafilepointer);
	if (!sizewritten) {
		messagewindow->value ("Error in writing data.");
		return 1;
	}
	
	return 0;
}